<?php

use Drupal\migrate\MigrateSkipRowException;
use Drupal\cforge\EventSubscriber\MigrationSubscriber;
use Drupal\user\Entity\User;
use Drupal\migrate\Row;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\cf_neighbourhoods\Entity\Neighbourhood;

 /**
  * Implements hook_migration_plugins_alter().
  */
function _cforge_migration_plugins_alter(array &$definitions)  {
  //when this runs a subsequent time, previously exported items are counted twice.
  foreach (array_keys($definitions) as $key) {
    if (substr($key, 0, 8) == 'upgrade_') {
      unset($definitions[$key]);
    }
  }

  $definitions['d7_block']['process']['region']['map']['sky_seldulac']['sky_seldulac'] = [
    'help' => 'top',
    'headerright' => null,
    'navigation' => 'top',
    'contentbottom' => 'content',
    'contentfooter' => 'footer',
    'footer' => null
  ];
  $definitions['d7_filter_format']['process']['filters']['id']['map'] = MigrationSubscriber::FILTER_FORMATS;
  $definitions['d7_filter_format']['process']['filters']['id']['default_value'] = 'plain_text';

  echo '_cforge_migration_plugins_alter: ';
  print_r($definitions['d7_menu_links']['migration_dependencies']);
  //in 8.4-rc2 menu items are failing validation because their routes haven't been migrated yet
  $definitions['d7_menu_links']['migration_dependencies']['required'][] = 'd7_node';
  $definitions['d7_menu_links']['migration_dependencies']['required'][] = 'd7_contact';
  $definitions['d7_comment']['migration_dependencies']['required'][] = 'd7_user';

  // Simplify the comment type process, which is confused by the renamings of things.
  // Then see
  //$definitions['d7_comment']['process']['comment_type'] = 'comment_type';

  foreach ($definitions as &$def) {
    if (isset($def['process']['event_date'])) {
      $def['process']['date'] = $def['process']['event_date'];
      $def['process']['date']['process']['value']['to_format'] = 'Y-m-d\TH:i:s';
      unset($def['process']['event_date']);
    }
  }

  $definitions['d7_file']['source']['constants']['source_base_path'] = DRUPAL_ROOT;
  $definitions['d7_menu']['process']['id']['bypass'] = 'FALSE';//don't import any unknown menus
  $definitions['d7_user']['migration_dependencies']['required'][] = 'd7_cf_neighbourhoods';

  $skip = [
    'action_settings',
    'user_profile_field',
    'user_profile_field_instance',
    'user_profile_entity_display',
    'user_profile_entity_form_display',
    'd7_view_mode',
    'd7_search_settings',
    'd7_search_page',
    'd7_comment_entity_form_display',
    'd7_comment_entity_display',
    'd7_comment_field_instance',
    'd7_system_file', 'd7_system_mail', 'd7_theme_settings'
  ];
  foreach ($skip as $mig) {
    unset($definitions[$mig]);
  }
  unset(
    $definitions['d7_filter_format']['process']['filters']['id']['bypass'],
    $definitions['d7_user']['process']['user_picture']//after a week of trying I'm working around this.
  );
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_filter_format_prepare_row(Row $row, $source, $migration) {
  $ignore = array_keys(MigrationSubscriber::FILTER_FORMATS);
  if (in_array($row->getSourceProperty('format'), $ignore)) {
    throw new MigrateSkipRowException("Ignoring unknown filter format");
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_comment_prepare_row(Row $row, $source, $migration) {
  // Skip comments where the user has been deleted.
  $uid = $row->getSourceProperty('uid');
  if (!User::load($uid)) {
    throw new MigrateSkipRowException("User $uid no longer exists");
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_menu_links_prepare_row(Row $row, $source, $migration) {
  // Only migrate menu links which are not already in the menu tree
  //use the title to identify menu links
  $query = \Drupal::database()->select('menu_tree', 'mt')
    ->fields('mt');
  $translated = new TranslatableMarkup($title);
  $query->condition($query->orConditionGroup()
    ->condition('title', serialize((string)$translated))
    ->condition('title', serialize($title))
  );
  if ($query->execute()->fetch()) {
    throw new MigrateSkipRowException("Menu link '$title' already exists");
  }

  if ($row->getSourceProperty('router_path') == 'admin/structure/taxonomy/%') {
    if ($row->getSourceProperty('link_title')) {
      $vocab = end(explode('/', $row->getSourceProperty('link_path')));
      $row->setSourceProperty('link_path', 'admin/structure/taxonomy/manage/'.$vocab.'/overview');
      \Drupal::messenger()->addStatus('upgrading manu link_path '.$row->getSourceProperty('link_path'));
    }
  }
  if ($row->getSourceProperty('link_path') ==  'admin/config/system/backup_migrate/export') {
    $row->setSourceProperty('link_path', 'admin/config/development/backup_migrate');
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_theme_settings_prepare_row(Row $row, $source, $migration) {
  $active_theme_name = \Drupal::theme()->getActiveTheme()->getName();
  if ($row->getSourceProperty('name') != 'theme_'.$active_theme_name.'_settings') {
    throw new MigrateSkipRowException("Active theme is $active_theme_name; Not migrating ".$row->getSourceProperty('name'));
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_user_prepare_row(Row $row, $source, $migration) {
  if ($row->getSourceProperty('uid') == 0) {
    throw new MigrateSkipRowException("Don't migrate user 0");
  }
  $address = $row->getSourceProperty('profile_address');
  $address[0]['dependent_locality'] = cforge_migrate_lookup_neighbourhood($address[0]['dependent_locality']);
  $row->setSourceProperty('profile_address', $address);
}

/**
 * Get the new neighbourhood name from the old.
 * @param string $old
 * @return string
 */
function cforge_migrate_lookup_neighbourhood(string $old) : string {
  static $neighbourhoods = [];
  if (empty($neighbourhoods)) {
    foreach (Neighbourhood::loadMultiple() as $key => $hood) {
      $hoods[$hood->label()] = $key;
    }
  }
  return $hoods[$old];
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_contact_category_prepare_row(Row $row, $source, $migration) {
  // The category is a machine name taken from the localised d7 category name.
  if (strlen($row->getSourceProperty('category')) > 32) {
    throw new MigrateSkipRowException('Skipping contact category '.$row->id());
  }
}

/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_block_prepare_row(Row $row, $source, $migration) {
  $ignoreDeltas = ['visitors', 'secondary-menu', 'form', 'online'];// the last one is the search form
  $ignoreThemes = ['bartik', 'seven'];
  if (in_array($row->getSourceProperty('delta'), $ignoreDeltas) or in_array($row->getSourceProperty('theme'), $ignoreThemes)) {
    throw new MigrateSkipRowException('Skipping block in theme '.$row->getSourceProperty('theme'));
  }
}

/**
 * Implements hook_migrate_prepare_row().
 * @todo
 */
function cforge_migrate_d7_action_prepare_row(Row $row, $source, $migration) {
  $callback = $row->getSourceProperty('callback');
  if (substr($callback, 0, 22) == 'views_bulk_operations_') {
    if (!$this->moduleHandler->moduleExists('views_bulk_operations')) {
      throw new MigrateSkipRowException();
    }
  }
  elseif (!is_callable($callback)) {
    throw new MigrateSkipRowException('Callback not found: '.$callback);
  }
  $skip = [
    'mcapi_transaction_mail_action',
    'og_set_state_action',
    'og_membership_delete_action',
    'system_block_ip_action',
    'views_bulk_operations_user_roles_action',
    'views_bulk_operations_archive_action',
    'views_bulk_operations_argument_selector_action',
    'views_bulk_operations_delete_item',
    'views_bulk_operations_delete_revision',
    'views_bulk_operations_modify_action',
    'views_bulk_operations_remove_from_book_action'
  ];
  if (in_array($row->id(), $skip)) {
    throw new MigrateSkipRowException('Skipping action '.$row->id());
  }
}


/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_field_prepare_row(Row $row, $source, $migration) {
  $fname = $row->getSourceProperty('field_name');
  // also d7 field location maps to user__coordinates in module cforge_geo
  if (substr($fname, 0, 3 == 'og_')) {
    throw new MigrateSkipRowException('Skipping og field '.$fname);
  }
  $existing_fields = [
    'account_offline', // Now a user data field @todo
    'manage_responsibility',// Abandoned.
    'offers_wants_types',// Abandoned
    'profile_address',// changed its name
    'profile_phones',// changed its name
    'profile_notes',// Abandoned
    'profile_location',// Abandoned
    'manage_notes',// Abandoned
  ];
  if (in_array($fname, $existing_fields)) {
    throw new MigrateSkipRowException("Field $fname has changed its name: ".$migration->id());
  }
}


/**
 * Implements hook_migrate_prepare_row().
 */
function cforge_migrate_d7_field_instance_prepare_row(Row $row, $source, $migration) {
  // See also cforge_migrate_d7_field_prepare_row().
  $existing_fields = [
    'account_offline', // Now a user data field @todo
    'manage_responsibility',// Abandoned.
    'offers_wants_types'// Abandoned
  ];
  if (in_array($row->getSourceProperty('field_name'), $existing_fields)) {
    return NULL;
  }
}