<?php

namespace Drupal\alt_login\Plugin\AltLoginMethod;

use Drupal\alt_login\AltLoginMethodInterface;
use Drupal\user\UserInterface;
use Drupal\user\Entity\User;
use Drupal\Core\Entity\Query\Sql\Condition;

/**
 * Plugin implementation for logging in with the user name as an alias.
 *
 * @AltLoginMethod(
 *   id = "uid",
 *   label = @Translation("User ID"),
 *   description = @Translation("Drupal's database key")
 * )
 */
class Uid extends Username implements AltLoginMethodInterface {

  /**
   * {@inheritDoc}
   */
  function applies($alias) : bool{
    $num = (int)$alias;
    return $num == $alias;
  }

  /**
   * {@inheritDoc}
   */
  function getUserFromAlias($alias) : UserInterface{
    if ($user = User::load($alias)) {
      return $user;
    }
  }

  /**
   * {@inheritDoc}
   */
  function getAlias(UserInterface $user) : string {
    return $user->id();
  }

  /**
   * {@inheritDoc}
   */
  function entityQuery(Condition $or_group, $match) {
    if (is_numeric($match)) {
      $or_group->condition('uid', $match);
    }
  }
}
