<?php

namespace Drupal\cf_neighbourhoods\Plugin\migrate\destination;

use Drupal\cf_neighbourhoods\Entity\Neighbourhood;
use Drupal\migrate\Row;
use Drupal\migrate\Plugin\migrate\destination\EntityConfigBase;

/**
 * Provides entity destination plugin.
 *
 * @MigrateDestination(
 *   id = "cf_neighbourhoods"
 * )
 */
class Neighbourhoods extends EntityConfigBase {

  //protected $supportsRollback = FALSE;

  /**
    * {@inheritdoc}
    */
  public function import(Row $row, array $old_destination_id_values = []) {
    $hood_list = $row->getSourceProperty('cforge_neighbourhoods');
    if (empty($hood_list)) {
      \Drupal::messenger()->addWarning('No Neighbourhoods found to migrate: '.implode('; ', $row->getSource()));
      return [];
    }
    foreach (explode("\n", $hood_list) as $hood) {
      $hood = trim($hood);
      if (empty($hood)) {
        continue;
      }
      if ($hoods = $this->storage->loadByProperties(['label' => $hood])) {
        $neighbourhood[] = reset($hoods);
      }
      else {
        $neighbourhood = Neighbourhood::create(['label' => $hood])->save();
      }
    }
    //just return the last nid because only one is expected
    return [$neighbourhood->id()];
  }

  /**
   * {@inheritDoc}
   */
  public function getIds() {
    return [
      'id' => [
        'type' => 'string',
        'max_length' => 8,
        'is_ascii' => TRUE,
      ],
    ];
  }


  protected static function getEntityTypeId($plugin_id) {
    return 'neighbourhood';
  }


}
