<?php

namespace Drupal\mcapi_bursers\Plugin\Field\FieldType;

use Drupal\Core\Field\EntityReferenceFieldItemList;

/**
 * Represents a Burser reference field list
 */
class BurserReferenceFieldItemList extends EntityReferenceFieldItemList {

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    // Support passing in only the value of the first item, either as a literal
    // (value of the first property) or as an array of properties.
    if (isset($values) && (!is_array($values) || (!empty($values) && !is_numeric(current(array_keys($values)))))) {
      $values = [$values];
    }

    // If the first value is not the wallet's owner, then shunt it in.
    $walletOwnerId = $this->getEntity()->getOwnerId();
    foreach ($values as $delta => $value) {
      if ($value['target_id'] == $walletOwnerId) {
        unset($values[$delta]);
        \Drupal::messenger()->addWarning($this->t("Wallet holder @name has been removed as a burser."));
        break;
      }
    }

//    if (empty($values[0]['target_id']) or $values[0]['target_id'] != $walletOwnerId) {
//      array_unshift($values, ['target_id' => $walletOwnerId]);
//    }
    $this->unique($values);
    parent::setValue($values, $notify);
  }

  /**
   * {@inheritdoc}
   *
   * The default value is the wallet owner
   *
   * @note On new wallets this didn't work because we can't getEntity() while it
   * is still being created.
   */
  public function applyDefaultValue($notify = TRUE) {
//    if ($this->parent) {
      $wallet = $this->getEntity();
      debug($wallet->toArray());
      if ($entity_type = $wallet->holder_entity_type->value) {
        $target_entity_id = \Drupal::entityTypeManager()
          ->getStorage($entity_type)
          ->load($wallet->holder_entity_id->value)
          ->getOwnerId();
        parent::setValue(['target_id' => $target_entity_id], $notify);
        \Drupal::messenger()->addWarning("Applying default burser value ($target_entity_id) to wallet ".$wallet->label());
      }
//    }
    return $this;
  }

  private function unique(&$array) {
    $array = array_map('json_encode', $array);
    $array = array_unique($array);
    foreach ($array as $key => $item) {
      $array[$key] = (array)json_decode($item, JSON_OBJECT_AS_ARRAY);
    }
  }

}
