<?php

namespace Drupal\mcapi\Plugin\views\field;

use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Field handler to display entity label optionally linked to entity page.
 *
 * @ViewsField("mcapi_wallet_entity_label")
 */
class WalletLabel extends FieldPluginBase {


  public function query() {
    // $this->addAdditionalFields();
  }

    /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['link_to_entity'] = ['default' => FALSE];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $form['link_to_entity'] = [
      '#title' => $this->t('Link to entity'),
      '#description' => $this->t('Make entity label a link to entity page.'),
      '#type' => 'checkbox',
      '#default_value' => !empty($this->options['link_to_entity']),
    ];
    parent::buildOptionsForm($form, $form_state);
  }


  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
     $wallet = \Drupal\mcapi\Entity\Wallet::load($values->wid);

    if (!empty($this->options['link_to_entity'])) {
      try {
        $this->options['alter']['url'] = $wallet->toUrl();
        $this->options['alter']['make_link'] = TRUE;
      }
      catch (UndefinedLinkTemplateException $e) {
        $this->options['alter']['make_link'] = FALSE;
      }
      catch (EntityMalformedException $e) {
        $this->options['alter']['make_link'] = FALSE;
      }
    }

    return $this->sanitizeValue($wallet->label());
  }

}
