<?php

namespace Drupal\mcapi\Storage;

use Drupal\mcapi\Entity\Wallet;
use Drupal\user\Entity\User;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;


/**
 * Storage controller for wallet entity.
 */
class WalletStorage extends SqlContentEntityStorage implements WalletStorageInterface {

  /**
   * {@inheritdoc}
   */
  public function getQueryServiceName() {
    return 'mcapi_wallet.query.sql';
  }

  /**
   * {@inheritdoc}
   */
  public static function walletsOf(ContentEntityInterface $entity, $load = FALSE) {
    $entity_type_id = $entity instanceOf ContentEntityInterface ? $entity->getEntityTypeId() : 'user';
    // There's no elegant static way to get an entityType's entityQuery object
    // or storage
    $wids = \Drupal::entityQuery('mcapi_wallet')
      ->condition('holder_entity_type', $entity_type_id)
      ->condition('holder_entity_id', $entity->id())
      ->execute();

    return $load ?
      Wallet::loadMultiple($wids) :
      $wids;
  }

  /**
   * {@inheritdoc}
   */
  public static function myWallets(AccountInterface $user) {
    if (!$user instanceof ContentEntityInterface) {
      $user = User::load($account->id());
    }
    return self::walletsOf($user);
  }

  /**
   * Return the first wallet held by a given entity.
   *
   * @param ContentEntityBase $entity
   *   Defaults to the current user.
   * @return int
   *   0 if no wallets are held
   */
  static function firstWalletOfEntity(ContentEntityInterface $entity = NULL) {
    if (!$entity) {
      $entity = \Drupal::currentUser();
    }
    if ($wids = self::walletsOf($entity)) {
      return reset($wids);
    }
    return 0;
  }

}

