<?php

namespace Drupal\cforge\Controller;

use \Drupal\Core\Controller\ControllerBase;
use \Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\Cache;

/**
 * Displays the text of the readme file for the given modue.
 */
class BrokenLinks extends ControllerBase {

  private $cache;
  private $entityQueryFactory;

  function __construct($entity_query_factory, CacheBackendInterface $cache_backend) {
    $this->entityQueryFactory = $entity_query_factory;
    $this->cache = $cache_backend;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.query'),
      $container->get('cache.default')
    );
  }


  /**
   * Page callback.
   */
  public function Page() {
    $renderable = [];
    foreach ($this::FindBrokenLinks() as $entity_type => $type_info) {
      foreach ($type_info as $entity_id => $entity_info) {
        $entity = $this->entityTypeManager()->getStorage($entity_type)->load($entity_id);
        $renderable[]['#markup'] = $this->t(
          '@entity_link has broken links: @edit',
          [
            '@entity_link' => $entity->toLink(NULL, 'canonical', ['attributes'=> ['target' => '_blank']])->toString(),
            '@edit' => $entity->toLink(t('edit'), 'edit-form', ['attributes'=> ['target' => '_blank']])->toString()
          ]
        );
        $links = [];
        foreach ($entity_info as $uri => $err) {
          $url = Url::fromUri($uri)->setOption('attributes', ['target' => '_blank']);
          $broken = $this->getLinkGenerator()->generate($uri, $url);
          $links[] = ['#markup' => (string)$broken . ' '.$err];
        }
        $renderable[] = ['#theme' => 'item_list', '#items' => $links, '#type' => 'ul'];
      }
    }
    return $renderable;
  }


  /**
   * What will trigger this? And send the mail and to whom?
   * @return array
   *   The errors indexed by keys on the field table
   */
  function FindBrokenLinks() {
    if ($cache = $this->cache->get('brokenLinks')) {
      $broken_links = $cache->data;
    }
    else {
      $results = $broken_links = $cache_tags = [];
      $fields = $this->entityTypeManager()
        ->getStorage('field_storage_config')
        ->loadByProperties(['module' => 'text']);
      foreach ($fields as $field) {
        $entity_type = $field->getTargetEntityTypeId();
        $eids = $this->entityQueryFactory->get($entity_type)
          ->condition($field->getName(), '%http%', 'LIKE')
          ->execute();
        foreach ($this->entityTypeManager()->getStorage($entity_type)->loadMultiple($eids) as $entity) {
          if (preg_match_all('/http(s)?:\/\/[\w.-]+(?:\.[\w\.-]+)+[\w.\/]+/', $entity->{$field->getName()}->value, $matches)) {
            foreach ($matches[0] as $url) {
              if (!isset($results[$url])) {
                $results[$url] = $this->testUrl($url);
              }
            }
            if ($results[$url]) {
              $broken_links[$entity_type][$entity->id()][$url] = $results[$url];
              $cache_tags = array_merge($cache_tags, $entity->getCacheTags());
            }
          }
        }
      }
      $this->cache->set('brokenLinks', $broken_links, Cache::PERMANENT, $cache_tags);
    }
    return $broken_links;
  }

  /**
   * Title callback.
   */
  public function title($module) {
    return t('Blah');
  }



  static function testUrl($url) {
    if (function_exists('curl_version')) {
      $c = curl_init($url);
      curl_setopt($c, CURLOPT_FOLLOWLOCATION, TRUE);
      curl_setopt($c, CURLOPT_TIMEOUT, 2);
      curl_setopt($c, CURLOPT_NOBODY, TRUE);
      curl_exec($c);
      return curl_getinfo($c)['http_code'] == '404' ? 404 : curl_error($c);
    }
  }
}
